% dynare code for Christiano discussion of LWZ(2013) Econometrica
%----------------------------------------------------------------
% 0. Housekeeping (close all graphic windows)
%----------------------------------------------------------------

close all;


%to reproduce the graphs and the numbers in the handout execute this code with pert = 1
%the equilibrium conditions of the model are derived in a handout that appears in the 
%zip file that this code is in.

 
pert=1;%if 1, perturb the frisch elasticity; 
       %if 0, perturb time series properties of vartheta
       %if 2, make autocorrelation of housing demand 0.90



%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

var
    
ce 
ch 
lambda_e 
lambda_h 
Le 
Lh 
K 
N 
w  
q_l 
mu 
R 
B 
I 
q_k  
Y
nu
z 
var_phi 
theta; % 17 variables and three shocks

varexo
e_z 
e_var_phi 
e_theta;

% phi is a shock on the land preference
% theta is the LVT ratio
% z is TFP shock

parameters
beta_e
beta_h
gamma_e
gamma_h 
alpha 
phi 
delta 
news
rho 
psi 
L_bar 
var_phi_bar
rho_z 
rho_var_phi 
rho_var_phi1
rho_theta 
sigma_z 
sigma_theta 
theta_bar
sigma_var_phi
om;

%----------------------------------------------------------------
% 2. Set the parameters
%---------------------------------------------------------------

gamma_h     = .4976;       % habit persistence for the firm
gamma_e     = .6584;       % habit persistence for the household
beta_e      = .9855;       % discount factor for the firm
beta_h      = .99427095;  % (1+lambda_a) * beta_e: discount factor for entrepreneur, lambda_a=0.0089
phi         = .0695;       % substitution between land and capital
delta       = .0368;       % depreciation
psi         = .0001;           % labor elasticity 
alpha       = .3;          % substitution between (Land * capital) and labor
L_bar       = 1;           % land supply (fixed)
var_phi_bar = 0.0457;    % This is mean of AR(1) oaf
 

rho         =3.8162; % This is set so as to make N=.25 at ss.

theta_bar   = 0.75;
om          = 0.1753;
news        = 0;%if 1, vartheta shock observed 4 periods in advance; 0, contemporaneous
rho_z        =.4263;
rho_var_phi  =.9997;
rho_var_phi1 = 0;
rho_theta    =.9805;

sigma_z       =.0042;
sigma_theta   =.0112;
sigma_var_phi =.0462;

%-----------------------------------------------------------
% Compute the steady state
%----------------------------------------------------------

[ce_ss,ch_ss,lambda_e_ss,lambda_h_ss,Le_ss,Lh_ss,K_ss,N_ss,w_ss,q_l_ss,mu_ss,R_ss,B_ss,I_ss,Y_ss]= ...
        sstate(gamma_h,gamma_e,beta_e,beta_h,phi,delta,psi,alpha,theta_bar,L_bar,var_phi_bar,rho);
fid=fopen('sstate.tex','w');
fprintf(fid,' steady state properties of the model, baseline parameterization\n')
fprintf(fid,' ce = %5.3f, ch = %5.3f, lambda_e = %5.3f, lambda_h = %5.3f, Le = %5.3f, Lh = %5.3f, K = %5.3f\n',ce_ss,ch_ss,lambda_e_ss,lambda_h_ss,Le_ss,Lh_ss,K_ss)
fprintf(fid,' N = %5.3f, w = %5.3f, q_l = %5.3f, mu = %5.3f, 400*(R-1) = %5.3f, B = %5.3f, I = %5.3f, Y = %5.3f\n', ...
    N_ss,w_ss,q_l_ss,mu_ss,400*(R_ss-1),B_ss,I_ss,Y_ss)

%----------------------------------------------------------------
% 3. Model
%---------------------------------------------------------------

model;

#f    = (om/2)*(I/I(-1)-1)^2;
#fp   = om*(I/I(-1)-1);
#fp1  = om*(I(+1)/I-1);
#F    = z*(Le(-1)^phi*K(-1)^(1-phi))^alpha*N^(1-alpha);
#MPL  = phi*alpha*F/Le(-1);
#MPN  = (1-alpha)*F/N;
#MPK  = (1-phi)*alpha*F/K(-1);
#F1   = z(+1)*(Le^phi*K^(1-phi))^alpha*N(+1)^(1-alpha);
#MPL1 = phi*alpha*F1/Le;
#MPK1 = (1-phi)*alpha*F1/K;
#thet = theta;

% Entrepreneur equations

%(1)
lambda_e=1/(ce-  gamma_e  *ce(-1)) - beta_e  *   gamma_e   /(ce(+1)-gamma_e*ce);

MPN=w;

q_l = beta_e * (lambda_e(+1)/lambda_e) * (MPL1 + q_l(+1)) + mu/lambda_e * thet * q_l(+1);

1 = mu/nu * thet * q_k(+1) + (beta_e/nu)* (lambda_e(+1)*MPK1 + nu(+1)*(1-delta) )  ;

%(5)
1/R = beta_e *(lambda_e(+1)/lambda_e) + mu/lambda_e;

K = (1-delta)*K(-1)+(1-f)*I;

nu*(1-f-fp*I/I(-1))-lambda_e+beta_e*nu(+1)*(I(+1)/I)^2*fp1=0;

B = thet * (q_l(+1)*Le+q_k(+1)*K);

q_k = nu/lambda_e;
    
% HH equations

% FOC for ch
%(10)
lambda_h=1/(ch-gamma_h*ch(-1)) - beta_h*gamma_h/(ch(+1)-gamma_h*ch);

% FOC fo N (11)
lambda_h*w=N^psi*rho;

% land pricing (12)
q_l=beta_h*(lambda_h(+1)/lambda_h)*q_l(+1)+var_phi/(lambda_h*Lh);

% bond pricing (13)
1/R=beta_h*(lambda_h(+1)/lambda_h);

% budget constraint (14)
ch+q_l*(Lh-Lh(-1))+B/R=w*N+B(-1);

% Market clearing

% good (15)
ce + ch + I = F;

% land market (16)
Le+Lh=L_bar;

% GDP (17)
Y=F;


% shock
log(z)=rho_z*log(z(-1))+sigma_z*e_z;

log(var_phi)=(1-rho_var_phi-rho_var_phi1)*log(var_phi_bar)+rho_var_phi*log(var_phi(-1))
    +rho_var_phi1*log(var_phi(-2))+sigma_var_phi*  news*e_var_phi(-4)+(1-news)*e_var_phi;

log(theta)=(1-rho_theta)*log(theta_bar)+rho_theta*log(theta(-1))+sigma_theta*e_theta;

end;

%----------------------------------------------------------------
% 4. Computation
%----------------------------------------------------------------

initval;
ce=ce_ss;
ch=ch_ss;
lambda_e=lambda_e_ss;
lambda_h=lambda_h_ss;
Le=Le_ss;
Lh=Lh_ss;
K=K_ss;
N=N_ss;
w=w_ss;
q_l=q_l_ss;
mu=mu_ss;
R=R_ss;
B=B_ss;
theta=theta_bar;
z=1;
var_phi=var_phi_bar;
nu=lambda_e;
q_k=1;
I=I_ss;
Y=z*(Le^phi*K^(1-phi))^alpha*N^(1-alpha);
end;

check;

shocks;
var e_z;
stderr .01;
var e_var_phi;
stderr .01;
var e_theta;
stderr .01;
end;

stoch_simul(irf=16,order=1,nograph);
figure('name','response to housing demand shock, baseline')
pltIRF(Y_e_var_phi,Y_ss,K_e_var_phi,K_ss,q_l_e_var_phi,q_l_ss, ...
    Le_e_var_phi,Le_ss,Lh_e_var_phi,Lh_ss,I_e_var_phi,I_ss,B_e_var_phi,B_ss, ...
    ce_e_var_phi,ch_e_var_phi,ce_ss,ch_ss,w_e_var_phi,w_ss,N_e_var_phi,N_ss,q_k_e_var_phi, ...
    R_e_var_phi,mu_e_var_phi,mu_ss,var_phi_e_var_phi,var_phi_bar,theta_bar,psi,gamma_h,gamma_e,1);


cons_base=(ch_e_var_phi+ce_e_var_phi)/(ch_ss+ce_ss);
inv_base=I_e_var_phi/I_ss;
hours_base=N_e_var_phi/N_ss;
GDP_base=Y_e_var_phi/(ch_ss+ce_ss+I_ss);
land_p_base=q_l_e_var_phi/q_l_ss;

sY_base=sqrt(sum(GDP_base.^2));
sI_base=sqrt(sum(inv_base.^2));
sC_base=sqrt(sum(cons_base.^2));
shours_base=sqrt(sum(hours_base.^2));
sland_p_base=sqrt(sum(land_p_base.^2));
CI_base=sum(cons_base.*inv_base)/(sI_base*sC_base);

fprintf(fid,' second moments, baseline model')
fprintf(fid,' std dev of log Y = %5.3f, std dev of log I = %5.3f, std dev of log C = %5.5f\n',sY_base,sI_base,sC_base)
fprintf(fid,' std dev, log C/std dev log Y = %5.3f, sd dev, log I/std dev log Y = %5.3f\n',sC_base/sY_base,sI_base/sY_base)
fprintf(fid,' std dev, log land/std dev log I = %5.3f\n',sland_p_base/sI_base)
fprintf(fid,' correlation, log C and log I = %5.3f\n',CI_base) 

land_p_base=q_l_e_var_phi/q_l_ss 
if pert == 1
   psi = 1;
elseif pert == 0
%perturbation on time series properties of land demand
   rho1=.95;
   rho2=.99;
   rho_var_phi=(rho1+rho2);
   rho_var_phi1=-rho1*rho2;
%AR(2): (1-rho1*L)*(1-rho2*L)=1-(rho1+rho2)*L+rho1*rho2*L^2
% so x_t = (rho1+rho2)*x_t-1 - rho1*rho2*x_t-2
elseif pert == 2
   rho_var_phi=0.90;
end


[ce_ss,ch_ss,lambda_e_ss,lambda_h_ss,Le_ss,Lh_ss,K_ss,N_ss,w_ss,q_l_ss,mu_ss,R_ss,B_ss,I_ss,Y_ss]= ...
        sstate(gamma_h,gamma_e,beta_e,beta_h,phi,delta,psi,alpha,theta_bar,L_bar,var_phi_bar,rho);

fprintf(fid,' steady state properties of the model, perturbed parameterization\n')
fprintf(fid,' ce = %5.3f, ch = %5.3f, lambda_e = %5.3f, lambda_h = %5.3f, Le = %5.3f, Lh = %5.3f, K = %5.3f\n',ce_ss,ch_ss,lambda_e_ss,lambda_h_ss,Le_ss,Lh_ss,K_ss)
fprintf(fid,' N = %5.3f, w = %5.3f, q_l = %5.3f, mu = %5.3f, 400*(R-1) = %5.3f, B = %5.3f, I = %5.3f, Y = %5.3f\n', ...
    N_ss,w_ss,q_l_ss,mu_ss,400*(R_ss-1),B_ss,I_ss,Y_ss)


initval;
ce=ce_ss;
ch=ch_ss;
lambda_e=lambda_e_ss;
lambda_h=lambda_h_ss;
Le=Le_ss;
Lh=Lh_ss;
K=K_ss;
N=N_ss;
w=w_ss;
q_l=q_l_ss;
mu=mu_ss;
R=R_ss;
B=B_ss;
theta=theta_bar;
z=1;
var_phi=var_phi_bar;
nu=lambda_e;
q_k=1;
I=I_ss;
Y=z*(Le^phi*K^(1-phi))^alpha*N^(1-alpha);
end;

stoch_simul(irf=16,order=1,nograph);

figure('name','response to housing demand shock, perturbed')
pltIRF(Y_e_var_phi,Y_ss,K_e_var_phi,K_ss,q_l_e_var_phi,q_l_ss, ...
    Le_e_var_phi,Le_ss,Lh_e_var_phi,Lh_ss,I_e_var_phi,I_ss,B_e_var_phi,B_ss, ...
    ce_e_var_phi,ch_e_var_phi,ce_ss,ch_ss,w_e_var_phi,w_ss,N_e_var_phi,N_ss,q_k_e_var_phi, ...
    R_e_var_phi,mu_e_var_phi,mu_ss,var_phi_e_var_phi,var_phi_bar,theta_bar,psi,gamma_h,gamma_e,1);

cons=(ch_e_var_phi+ce_e_var_phi)/(ch_ss+ce_ss);
inv=I_e_var_phi/I_ss;
hours=N_e_var_phi/N_ss;
GDP=Y_e_var_phi/(ch_ss+ce_ss+I_ss);
land_p=q_l_e_var_phi/q_l_ss;

sY=sqrt(sum(GDP.^2));
sI=sqrt(sum(inv.^2));
sC=sqrt(sum(cons.^2));
shours=sqrt(sum(hours.^2));
sland_p=sqrt(sum(land_p.^2));
CI=sum(cons.*inv)/(sI*sC);
fprintf(fid,' second moments, perturbed model')
fprintf(fid,' std dev of log Y = %5.3f, std dev of log I = %5.3f, std dev of log C = %5.5f\n',sY,sI,sC)
fprintf(fid,' std dev, log C/std dev log Y = %5.3f, sd dev, log I/std dev log Y = %5.3f\n',sC/sY,sI/sY)
fprintf(fid,' std dev, log land/std dev log I = %5.3f\n',sland_p/sI)
fprintf(fid,' correlation, log C and log I = %5.3f\n',CI)  
figure('name','for discussion')
tt=1:16;
subplot(221)
plot(tt,100*cons(tt),tt,100*cons_base(tt),'*-',tt,zeros(length(tt),1))
if pert == 1
   legend(' Frisch elasticity = 1',' Frisch elasticity = infinity (as in paper)')
elseif pert == 0
   legend(' hump shaped AR(2) representation of housing preference',' Baseline AR(1) representation (0.9997) of housing preference')
elseif pert == 2
   legend(' AR(1) parameter = 0.9',' Baseline AR(1) (=0.9997) parameter for representation of housing preference')
end
title('consumption','FontSize',18)
ylabel('% deviation from ss','FontSize',18)
axis tight
subplot(222)
plot(tt,100*inv(tt),tt,100*inv_base(tt),'*-')
title('investment','FontSize',18)
ylabel('% deviation from ss','FontSize',18)
axis tight
subplot(223)
plot(tt,100*hours(tt),tt,100*hours_base(tt),'*-')
title('hours','FontSize',18)
ylabel('% deviation from ss','FontSize',18)
axis tight
subplot(224)
plot(tt,100*land_p(tt),tt,100*land_p_base(tt),'*-')
ylabel('% deviation from ss','FontSize',18)
title('land price','FontSize',18)
axis tight
str=['Response of Economy to a One Standard Deviation Positive Shock in Housing Demand'];
suptitle(str);

type sstate.tex
